﻿// The Nova Project by Ken Beckett.
// Copyright (C) 2007-2011 Inevitable Software, all rights reserved.

// This file contains test code for the Nova.CodeDOM library - it's not designed to be
// executed, only parsed and resolved.

using System;

namespace TestLibrary
{
    /// <summary>
    /// Test enum 1.
    /// </summary>
    public enum Enum1 : short
    {
        /// <summary>
        /// Enum member One.
        /// </summary>
        One,
        /// <summary>
        /// Enum member Two.
        /// </summary>
        Two
    }

    /// <summary>
    /// Test class 1.
    /// </summary>
    public class Test1 : IComparable
    {
        /// <summary>
        /// Enum field.
        /// </summary>
        public Enum1 _enum1;

        /// <summary>
        /// Volatile field.
        /// </summary>
        public volatile int _volatile;

        /// <summary>
        /// Unsafe field.
        /// </summary>
        public unsafe int* _unsafePointer;

        /// <summary>
        /// CompareTo
        /// </summary>
        public int CompareTo(object obj) { return 0; }

        /// <summary>
        /// Inner Test class.
        /// </summary>
        /// <typeparam name="T">Type parameter.</typeparam>
        public class InnerTest1<T> where T : struct, IComparable
        {
            /// <summary>
            /// Generic method.
            /// </summary>
            public W GenericMethod<U, V, W>(T p1, U p2, V p3)
                where U : class, ICloneable, new()
                where V : Test1, IComparable
                where W : V
            {
                return null;
            }
        }
    }

    /// <summary>
    /// Test struct 1.
    /// </summary>
    public struct Struct1
    { }

    /// <summary>
    /// Test unsafe class.
    /// </summary>
    public unsafe class UnsafeClass
    {
        /// <summary>
        /// Method that does unsafe things.
        /// </summary>
        public void DoUnsafeStuff()
        {
            Console.WriteLine("sizeof(Type)=" + sizeof(Struct1));
        }
    }

    /// <summary>
    /// Test safe class with unsafe method.
    /// </summary>
    public class SafeClass
    {
        /// <summary>
        /// Method that does unsafe things.
        /// </summary>
        public unsafe void DoUnsafeStuff()
        {
            Console.WriteLine("sizeof(Type)=" + sizeof(Struct1));
        }
    }

    /// <summary>
    /// Base test class.
    /// </summary>
    public class BaseTest : ICloneable
    {
        /// <summary>
        /// Base field to be hidden by derived class.
        /// </summary>
        public int Field;

        /// <summary>
        /// Base method to be overridden.
        /// </summary>
        public virtual void OverriddenMethod() { Console.WriteLine("BaseTest"); }

        /// <summary>
        /// Base method to be hidden by derived class.
        /// </summary>
        public void HiddenMethod() { Console.WriteLine("BaseTest"); }

        /// <summary>
        /// Clone method.
        /// </summary>
        public object Clone() { return MemberwiseClone(); }

        /// <summary>
        /// Base nested class to be hidden.
        /// </summary>
        public class NestedClass { }
    }

    /// <summary>
    /// Derived test class.
    /// </summary>
    public class DerivedTest : BaseTest, IDisposable, ICloneable
    {
        /// <summary>
        /// Derived field that hides same field in base.
        /// </summary>
        public new int Field;

        /// <summary>
        /// Derived method that overrides virtual method in base.
        /// </summary>
        public override void OverriddenMethod() { Console.WriteLine("DerivedTest"); }

        /// <summary>
        /// Derived method that hides same method in base.
        /// </summary>
        public new void HiddenMethod() { Console.WriteLine("DerivedTest"); }

        /// <summary>
        /// Dispose method.
        /// </summary>
        public void Dispose() { }

        /// <summary>
        /// Derived nested class that hides one in the base.
        /// </summary>
        public new class NestedClass { }
    }

    /// <summary>
    /// Test type that will conflict with System.Action (a delegate).
    /// </summary>
    public class Action
    {
    }
}
